#include "headio.h"
#include <ctype.h>
#include "Headped.H"
#include "headrandom.h"
#include <math.h>
#include <stdlib.h>
#include "util.h"

extern struct mar *nextmar(void);
void tryswitching(int ntries, float boiling, float cooler, 
		  float spacer, float yunit);
int *ningen;
int **marpic;

void annealcoords(int nit, float boil, float cool)
{
  /* 
   Uses the annealing algorithm to minimise the total horizontal components of 
     line length for a marriage node graph of the pedigree.
     The perturbation scheme is to swap the coordinates of two marriages and 
     adjust the individuals involved accordingly.
  */

  {
    struct mar *m;
    int i, j, a, nin, oldpt, most;
    float spacer, yunit;
    ningen = (int *)calloc(higen+1,sizeof(int));
    marpic = (int **)calloc(higen+1,sizeof(int *));
    yunit = .5/higen;

    for (i=1; i<=marpt; i++) {
      a = MAXIMUM(marstack[i].pa->info.gen,marstack[i].ma->info.gen);
      ningen[a]++;
    }
    for (i=1, most=0; i<higen; i++){
      marpic[i] = (int *)calloc(2*(ningen[i]+1),sizeof(int));

      if (most < ningen[i]) most = ningen[i];
      ningen[i] = 0;
    }
    spacer = 1.0/((most+1.0)*8.0);
    for (i=1; i<=marpt; i++){
      a = MAXIMUM(marstack[i].pa->info.gen,marstack[i].ma->info.gen);
      marpic[a][++ningen[a]] = i;
    }


    oldpt=marpt;

    for (i=1; i<higen; i++) {

	nin = ningen[i];
	for (j=nin+1; j<=2*nin; j++) {
	  ningen[i]++;
	  m = nextmar();
	  marpic[i][ningen[i]] = marpt;
	  m->info.ycoord = marstack[marpic[i][1]].info.ycoord;
	  a = marpic[i][ningen[i]-nin];
	  m->info.xcoord = marstack[a].info.xcoord;
	  m->info.xcoord += 1.0/(2*(nin+1.0));
	}
      
    }
    marpt = oldpt;
    tryswitching(nit,boil,cool,spacer,yunit);   
  }
}
 /*======================*/
  /* Annealing procedures */
  /*======================*/

#define INDCOORDS(X) \
{\
if ((X)->own != nomarlist)\
  {\
  float tot = 0.0;\
  int num = 0;\
  struct marlist *ml;\
  for (ml = (X)->own; ml != nomarlist; ml = ml->next)\
    {\
    tot += ml->cur->info.xcoord;\
    num ++;\
    }\
  (X)->info.xcoord = tot/num;\
  if ((X)->par != nomar)\
    {\
    (X)->info.xcoord = ((X)->info.xcoord + (X)->par->info.xcoord) / 2.0 ;\
    }\
  }\
else\
  (X)->info.xcoord = (X)->par->info.xcoord;\
}

#define DISTABOUT(X) \
{\
distab = 0.0;\
if ((X)->par != nomar)\
  {\
  xt = (X)->info.xcoord - (X)->par->info.xcoord;\
  distab += xt*xt ;\
  }\
for (ml = (X)->own; ml != nomarlist; ml = ml->next)\
  {\
  xt = (X)->info.xcoord - ml->cur->info.xcoord;\
  distab += xt*xt ;\
  }\
}

#define COPYANDDEC(A) \
{\
newpt++;\
indstack[newpt].pa = A;\
indstack[newpt].info.xcoord = A->info.xcoord;\
indstack[newpt].info.ycoord = A->info.ycoord;\
DISTABOUT(A)\
curtot -= distab;\
}

#define SWITCHCOORDS(A,B) \
{\
xt = A->info.xcoord;\
yt = A->info.ycoord;\
A->info.xcoord = B->info.xcoord;\
A->info.ycoord = B->info.ycoord;\
B->info.xcoord = xt;\
B->info.ycoord = yt;\
if (A->pa != noind)\
  {\
  if (A->pa->info.degree > 1) INDCOORDS(A->pa)\
  if (A->ma->info.degree > 1) INDCOORDS(A->ma)\
  for( il = A->kids; il != noindlist; il = il->next)\
    if (il->cur->info.degree > 1) INDCOORDS(il->cur)\
  }\
if (B->pa != noind)\
  {\
  if (B->pa->info.degree > 1) INDCOORDS(B->pa)\
  if (B->ma->info.degree > 1) INDCOORDS(B->ma)\
  for( il = B->kids; il != noindlist; il = il->next)\
    if (il->cur->info.degree > 1) INDCOORDS(il->cur)\
  }\
}


/*===================*/
/* Annealing program */
/*===================*/
void tryswitching(int ntries, float boiling, float cooler, 
		  float spacer, float yunit)
{

/* Annealing parameters */
int nup = 0, ndown = 0, nacross = 0, lastup = 0, lastdown = 0, lastacross = 0;
 float temp = boiling;
float freezing=0.0;
float curtot = 0, oldtot;


/* Indexes */
int i, j, c, d, e;
int newpt;
int ndiscon;
struct marlist *ml;
struct indlist *il;
struct mar *a, *b;
float xt, yt, distab;


/* Calculate initial length */
for (i=1; i<=indpt; i++) 
  if (indstack[i].info.degree > 1) { DISTABOUT(&indstack[i]) curtot += distab; }

/* Output initial data */
fprintf(stderr,"\n\n Searching for a minimum total squared line length graph \n");
fprintf(stderr,"\n Total squared x distances at start       = %12.5f",curtot);
fprintf(stderr,"\n Starting temperature                     = %12.5f",boiling);
fprintf(stderr,"\n Cooling factor                           = %12.5f",cooler);
fflush(stderr);
fprintf(stderr,"\n ");

/* Main loop */
 randinit();
for (i = 1; i <= ntries; i++, temp *= cooler)
  {

  /* Select marriages to switch */
  do { c = randint(1,higen-1); } while (ningen[c] < 2);
  do
    {
    d = randint(1,ningen[c]);
    do { e = randint(1,ningen[c]); } while ( e == d );
    d = marpic[c][d];
    e = marpic[c][e];
    }
  while ( d > marpt && e > marpt );
  a = &marstack[d];
  b = &marstack[e];

  /* Switch the marriages */
  oldtot = curtot;
  newpt = indpt;
  if (a->pa != noind)
    {
    if (a->pa->info.degree > 1) COPYANDDEC(a->pa)
    if (a->ma->info.degree > 1) COPYANDDEC(a->ma)
      for (il= a->kids; il != noindlist; il = il->next)
      if (il->cur->info.degree > 1) COPYANDDEC(il->cur)
      }
  if (b->pa != noind)
    {
    if (b->pa->info.degree > 1) COPYANDDEC(b->pa)
    if (b->ma->info.degree > 1) COPYANDDEC(b->ma)
     for (il= b->kids; il != noindlist; il = il->next)
      if (il->cur->info.degree > 1) COPYANDDEC(il->cur)
      }
  SWITCHCOORDS(a,b)
  if (a->pa != noind)
    {
    if (a->pa->info.degree > 1) { DISTABOUT(a->pa) curtot += distab; }
    if (a->ma->info.degree > 1) { DISTABOUT(a->ma) curtot += distab; }
    for (il= a->kids; il != noindlist; il = il->next)
      if (il->cur->info.degree > 1) { DISTABOUT(il->cur) curtot += distab; }
    }
  if (b->pa != noind)
    {
    if (b->pa->info.degree > 1) { DISTABOUT(b->pa) curtot += distab; }
    if (b->ma->info.degree > 1) { DISTABOUT(b->ma) curtot += distab; }
    for(il= b->kids;il != noindlist;il = il->next)
      if (il->cur->info.degree > 1) { DISTABOUT(il->cur) curtot += distab; }
    }

  /* Acceptance criterion */
  if (curtot < oldtot)
    { lastdown = i; ndown++; }
  else if (curtot == oldtot) 
    {lastacross = i; nacross++; }
  else if (curtot  <= oldtot - temp*log(randu01()))
    { lastup = i; freezing = temp; nup++; }
  else 
    {
    curtot = oldtot;
    xt = a->info.xcoord;
    yt = a->info.ycoord;
    a->info.xcoord = b->info.xcoord;
    a->info.ycoord = b->info.ycoord;
    b->info.xcoord = xt;
    b->info.ycoord = yt;
    for (j = indpt + 1; j <= newpt; j++)
      {
      indstack[j].pa->info.xcoord = indstack[j].info.xcoord;
      indstack[j].pa->info.ycoord = indstack[j].info.ycoord;
      }
    }
  }

/* End of main loop */

/* Output results */
fprintf(stderr,"\n Total squared x distances after searching= %12.5f",curtot);
fprintf(stderr,"\n Finishing temperature                    = %12.5f",temp);
fprintf(stderr,"\n Freezing temperature                     = %12.5f",freezing);
fprintf(stderr,"\n Number of tries                          = %8d",ntries);
fprintf(stderr,"\n Number of downhill steps                 = %8d",ndown);
fprintf(stderr,"\n Last downhill step                       = %8d",lastdown);
fprintf(stderr,"\n Number of uphill steps                   = %8d",nup);
fprintf(stderr,"\n Last uphill step                         = %8d",lastup);
fprintf(stderr,"\n Number of lateral steps                  = %8d",nacross);
fprintf(stderr,"\n Last lateral step                        = %8d\n",lastacross);

/* Set coords of all individuals */
for (i=1, ndiscon=1; i<=indpt; i++)
  if (indstack[i].info.degree == 0) ndiscon++;
  else if (indstack[i].info.degree < 2) setindcoords(&indstack[i],yunit);
for (i=1, j=0; i<=indpt; i++)
  if (indstack[i].info.degree == 0) 
    {
    j++;
    indstack[i].info.xcoord = 1.0*j/((float) ndiscon);
    indstack[i].info.ycoord = .03;
    }
    
for (i=1; i<=marpt; i++)
  if (marstack[i].pa->par == nomar || marstack[i].ma->par == nomar)
    if (marstack[i].pa->info.xcoord == marstack[i].ma->info.xcoord)
      { 
      if (marstack[i].pa->par == nomar) marstack[i].pa->info.xcoord -= spacer;
      if (marstack[i].ma->par == nomar) marstack[i].ma->info.xcoord += spacer;
      }
}






