/*============================================*/
/* select gene genealogy and make a list. */
/*============================================*/
#include "Headped.H"
#include <cstring>
#include <cctype>
#include "getopti.h"
#include <cstdlib>
#include <cmath>
#include <iostream>
#include <fstream>
#include <ctime>


#define UPINCH 72

#include "headio.h"
#include "util.h"
using namespace std;

struct ggdata{
  int name, parent, t;
  float x;
  int isterm;
  unsigned char r,g,b;
  char *label;
};

struct ggdata *gg;
int ggcnt, ggt;
float ggx;


static char version[]="0.6 (20070304) JC Loredo-Osti and K Morgan.";
void usage(char *foo){
  cout << "\n     Usage: " << foo << " [options] file\n"
       << "Options:\n"
       << "\t{ -o, --output }<filename>\n"
       << "\t{ -f, --female }    female gene genealogy\n"
       << "\t{ -m, --male }      make gene genealogy\n"
       << "\t{ -l, --labels }    include labels\n"
       << "\t{ -h, --height }<inches>    page height\n"
       << "\t{ -w, --width }<inches>     page width\n"
       << "\t{ -r, --rotate}     90 degree rotation\n"
       << "\t{ -M, --margins }<inches>\n"
       << "\t{ -b, --black }     black edges\n"
       << "\t{ -s, --show }      show defaults\n"
       << "\t{ -v, --version}\n"
       << endl;
}

void defaults(char *foo){
  cout << endl << foo << " takes the following"
       << " default actions:\n\n"
       << " female gene genealogy are extracted,\n"
       << " no labels, and individuals with unknown\n"
       << " sex are left out; bounding box is computed\n"
       << " for a 11x8.5 inch medium; no rotation;\n margins at 0.25 inches.\n"
       << endl;
  exit(0);
}

int nextcreature(int p, int cnt){
  int i;
  for(i = cnt; i <= ggcnt; i++)
    if(gg[i].parent == gg[p].name){
      return(i);
    }
  return 0;
}

void ggdatget(struct ind iig, int sex){
  int tmp=0;
  if(sex == MALE)
    tmp = iig.pa->info.name;
  if(sex == FEMALE)
    tmp = iig.ma->info.name;
  gg[ggcnt].name = iig.info.name;
  gg[ggcnt].parent = tmp;
  gg[ggcnt].r = iig.info.r;
  gg[ggcnt].g = iig.info.g;
  gg[ggcnt].b = iig.info.b;
  gg[ggcnt].label = iig.info.label;
}

void swapgg(int i, int j){
  int tmp;
  unsigned char c;
  char *ll;

  tmp=gg[i].name;
  gg[i].name = gg[j].name;
  gg[j].name = tmp;
  tmp=gg[i].t;
  gg[i].t = gg[j].t;
  gg[j].t = tmp;
  tmp=gg[i].parent;
  gg[i].parent = gg[j].parent;
  gg[j].parent = tmp;
  ll=gg[i].label;
  gg[i].label = gg[j].label;
  gg[j].label = ll;
  c=gg[i].r;
  gg[i].r = gg[j].r;
  gg[j].r = c;
  c=gg[i].g;
  gg[i].g = gg[j].g;
  gg[j].g = c;
  c=gg[i].b;
  gg[i].b = gg[j].b;
  gg[j].b = c;
}

void bubblegg(int to, int from){
  int i;
  for(i = from; i > to ; i--)
    swapgg(i, i-1);
}

void getggprole(int p){
  int i,cnt;
  cnt=p;
  while((i = nextcreature(p,++cnt))){
    if(i != cnt) bubblegg(cnt,i);
    gg[cnt].t = gg[p].t + 1;
  }
  if(gg[p].name != gg[p+1].parent) gg[p].isterm = 1;
}

float getggx(int p, int from, int to){
  float x1,x2;
  int i;
  x1 = x2 = 0.0;
  for(i = from; i < to; i++)
    if(gg[i].parent == p)
      x1 = gg[i].x;
  for(i = to -1; i >= from; i--)
    if(gg[i].parent == p)
      x2 = gg[i].x;
  return( (x1+x2)/2.0 );
}


void sortggdat(int sex, int jo){
  int i, i0;
  ggcnt = 0;
  for(i = 1; i <= indpt; i++){
    if(indstack[i].info.sex == sex){
      ggcnt++;
      ggdatget(indstack[i],sex);
    }
  }
  for(i=1; i<= ggcnt; i++){
    getggprole(i);
  }
  gg[ggcnt].isterm = 1;
  ggt = 0;
  for(i=1; i<=ggcnt; i++)
    if(ggt < gg[i].t) ggt = gg[i].t;

  i0 = 1;
  ggx = 1.0;
  for(i=1; i<=ggcnt+1; i++){
    int t, j;
    if(gg[i].parent == 0){
      for(t = ggt-1; t>= 0; t--)
	for(j = i0; j < i; j++)
	  if(gg[j].t == t && gg[j].x == 0) gg[j].x = getggx(gg[j].name, i0, i);
      i0 = i;
      ggx += 1.0;
    }
    if(gg[i].isterm){
      gg[i].x = ggx;
      ggx += 1.0;
    }
  }
  ggx += 1.0;
  ggt++;
  for(i=1; i<=ggcnt; i++) gg[i].t = ggt - gg[i].t;

}

void mkps(char *file, int p, float hei, float wid, bool rotate, float marg,
	  bool lab, bool coloured = 1, bool unks = 0){
  int i;
  float tmp, xscale, yscale, x0, y0;
  ofstream out;

  unsigned int edgec[7][3] = {
    {0,0,0},
    {150,0,0},
    {0,0,250},
    {100,75,25},
    {25,25,100},
    {50, 100, 200},
    {100, 50, 150}
  };
  int edgecolour = 1;

  if(file[0]){
    out.open(file);
    if(!out){
      cerr << " File '" << file << "' cannot be opened.\n"
	   << " Maybe you don't have rights in this directory." << endl;
      exit(1);
    }
    cout.rdbuf(out.rdbuf());
  }
  sortggdat(p,unks);
  xscale = floor(wid * UPINCH);
  yscale = floor(hei * UPINCH);
  time_t now = time((time_t *)NULL);

  cout << "%!PS-Adobe-3.0 EPSF-3.0\n"
       << "%%Creator: mkgg\n"
       << "%%CreationDate: " << ctime(&now)
       << "%%DocumentData: Clean7Bit\n"
       << "%%BoundingBox: 0 0 " << xscale << ' ' << yscale << "\n\n"
       << "%%BeginProlog" << endl;

  if(rotate)
    { tmp = xscale; xscale = yscale; yscale = tmp; }

  if((xscale/ggx) < marg ){
    xscale -= 2*marg;
    x0 = marg;
  }
  else
    x0 = 0;

  if((xscale/(ggt + 1)) < marg ){
    yscale -= 2*marg;
    y0 = marg;
  }
  else
    y0 = 0;

  xscale /= ggx;

  yscale /= (ggt + 1);
  
  tmp = xscale < yscale ? xscale : yscale;
  tmp = (tmp > 5 ? tmp : 5)/2.5;
  tmp = tmp <= 14 ? tmp : 14;
  cout << "/noderadius " << tmp << " def" << endl;
  tmp = 0.5 + 0.05*tmp;
  cout << "/linewidth " << tmp << " def" << endl;
  if(lab) 
    cout << "/lineshadow " << tmp+0.1 << " def" << endl;
  cout << "/sc {\n\t setrgbcolor\n } def" << endl;
  if(lab)
    cout << "/labelgap " << (4.0 * tmp) <<  " def" << endl;

  cout << "/node {\n\t ";
  if(lab) 
    cout << "5 copy symb\n\t count 6 ge"
	 << "\n\t {\n\t\t 2 noderadius mul sub labelgap sub\n"
	 << "\t\t 5 2 roll pop pop pop\n"
	 << "\t\t moveto dup stringwidth pop -2 div 0 rmoveto\n"
	 << "\t\t gsave lineshadow setlinewidth\n"
	 << "\t\t dup gsave 1 setgray false charpath stroke grestore\n"
	 << "\t\t 0 setgray show grestore }\n"
	 << "\t {\n\t pop pop pop pop pop\n\t } ifelse\n"
	 <<" } def" << endl
	 << "/symb {\n\t " ;
  cout << "newpath\n\t noderadius";
  if(p == FEMALE)
    cout << " 0 360 arc" << endl;
  else{
    cout << " 1.7725 mul /squlen exch def\n"
	 << "\t squlen -2 div /squhalf exch def\n"
	 << "\t squlen -1 mul /squneg exch def\n\t squhalf add moveto\n"
	 << "\t squhalf 0 rmoveto\n\t 0 squlen rlineto squlen 0 rlineto\n"
	 << "\t 0 squneg rlineto squneg 0 rlineto" << endl;
  }
  cout << "\t closepath\n \t gsave sc fill grestore\n"
       << "\t linewidth setlinewidth stroke\n } def" << endl;

  cout << "/edge {\n\t moveto lineto stroke\n } def" << endl;

  cout << "%%EndProlog\n" << endl;
  cout << "linewidth setlinewidth\n";
  if(!coloured) 
    cout << "0 0 1 sc" << endl;
  if(rotate)
    cout << floor(wid *UPINCH) << " 0 translate 90 rotate" << endl;
  cout << "%/Helvetica-Bold findfont 1.7 noderadius mul scalefont setfont\n"
       << "%40 40 moveto\n%";
  if(p == FEMALE)
    cout << "(Female";
  else
    cout << "(Male";
  cout << " gene genealogy) show" << endl;
  if(lab)
    cout << "/Helvetica findfont 1.2 noderadius mul scalefont setfont" << endl;
  for(i = 1; i<= ggcnt; i++)
    if(gg[i].parent){
      int j;
      for(j = i; j>=1; j--)
	if(gg[i].parent == gg[j].name) break;
	cout << (x0+gg[j].x*xscale) << ' '
	     << (y0+gg[j].t*yscale) << ' '
	     << (x0+gg[i].x*xscale) << ' '
	     << (y0+gg[i].t*yscale) << " edge" << endl;
    }else{
      if(coloured){
	if(gg[i+1].parent)
	  cout << edgec[edgecolour][0]/255.0 << ' '
	       << edgec[edgecolour][1]/255.0 << ' '
	       << edgec[edgecolour][1]/255.0 << ' '
	       << "sc" << endl;
	edgecolour++;
	edgecolour %= 7;
      }
    }
  edgecolour = 1;
  for(i = 1; i<= ggcnt; i++){
    if(!gg[i].parent){
      if(coloured){
	cout << edgec[edgecolour][0]/255.0 << ' '
	     << edgec[edgecolour][1]/255.0 << ' '
	     << edgec[edgecolour][1]/255.0 << ' '
	     << "sc" << endl;
	edgecolour++;
	edgecolour %= 7;
      }
    }
    if(lab)
      cout << '(' <<  gg[i].label << ") ";    
    cout << gg[i].r/255.0 << ' ' 
	 << gg[i].g/255.0 << ' '
	 << gg[i].b/255.0 << ' ';
    cout << (x0+gg[i].x*xscale) << ' '
	 << (y0+gg[i].t*yscale) << " node" <<endl;

  }
  cout << "showpage" << endl;
}

int main (int argc, char **argv)
{
  char outfile[maxstr], pedfilename[maxstr];
  FILE *file;
  int kk;
  int openpedc;
  bool labels = 0;
  bool impute = 0;
  bool rotation = 0;
  float he, wi, marg;
  static struct option _options[] =
  {
    {"output",1,'o'},
    {"male",0,'m'},
    {"female",0,'f'},
    {"labels",0,'l'},
    {"impute",0,'i'},
    {"height",1,'h'},
    {"width",1,'w'},
    {"rotate",0,'r'},
    {"margins",1,'M'},
    {"show",0,'s'},
    {"version",0,'v'},
    {0, 0, 0}
  };

  int p = 2;

  outfile[0]='\0';
  he = 11;
  wi = 8.5;
  marg = 0.25;
  GetOpt opto(argc, argv,"o:mflh:w:rsvM:",_options);
    
  while (1){
    kk = opto();
    if (kk == -1)
      break;
      
    switch (kk)
      {
      case 'o':
	if(! opto.optarg){usage(argv[0]); exit(1);}
	strcpy(outfile,opto.optarg);
	break;
	
      case 'm':
	p=1;
	break;

      case 'f':
	p=2;
	break;

      case 'l':
	labels = 1;
	break;

      case 'i':
	impute = 1;
	break;

      case 'r':
	rotation = 1;
	break;

      case 'h':
	if(!opto.optarg){usage(argv[0]); exit(1);}
	he = atof(opto.optarg);
	if(he < 1.0) he = 1.0;
	break;

      case 'w':
	if(!opto.optarg){usage(argv[0]); exit(1); }
	wi = atof(opto.optarg);
	if (wi < 1.0) wi = 1.0;
	break;

      case 'M':
	if(!opto.optarg){usage(argv[0]); exit(1); }
	marg = atof(opto.optarg);
	if (marg < 0.1) marg = 0.1;
	break;

      case 's':
	defaults(argv[0]);

      case 'v':
	printf("Version: %s\n",version);
	exit(0);

      case '?':
	usage(argv[0]);
	exit(1);
	break;
     
      default:
	usage(argv[0]);
	exit(0);
      }
  }

  if (opto.optind < argc){
    strcpy(pedfilename, argv[opto.optind++]);
    if ( (file = fopen(pedfilename,"r")) != NULL ){
      openpedc = inputpedc(file,0);
      fclose(file);
      if(!openpedc){
	cerr << "Error file: " << pedfilename
	     << " is not in \"pedc\" format." << endl;
	exit(1);
      }
      gg = (struct ggdata *) calloc((unsigned)maxind+2,sizeof(*indstack));
      mkps(outfile, p, he, wi, rotation,marg*UPINCH,labels);
    }
  }
  else{
    usage(argv[0]);
    exit(1);
  }
  exit (0);
}
