/*============================================*/
/* Drawing a pedigree to PostScript printers. */
/*============================================*/
#include "postscript.h"
#include "Headped.H"
#include <cstring>
#include "getopti.h"
#include <cstdlib>
#include "util.h"
	

#include <fstream>
#include <iostream>

using namespace std;

static char version[]="0.6 (20070304) JC Loredo-Osti and K Morgan.";
void usage(char *foo){
  cout << "\n     Usage: " << foo << " [options] file\n"
       <<"Options:\n"\
       << "\t{ -o, --output }<filename>\n"
       << "\t{ -t, --title }<title>      Page title\n"
       << "\t{ -n, --noids }		 Don't print IDs\n"
       << "\t{ -r, --rotate }            90 degrees rotation\n"
       << "\t{ -l, --labels }            Print labels\n"
       << "\t{ -i, --noicons }           Eliminates icons\n"
       << "\t{ -f, --noframe }           Eliminates frame\n"
       << "\t{ -H, --height }<in>        Page height\n"
       << "\t{ -W, --width }<in>         Page width\n"
       << "\t{ -m, --margins }<in>\n"
       << "\t{ -w, --linewidth }<pts>\n"
       << "\t{ -e, --elbow_room }<pts>\n"
       << "\t{ -x, --extra_margin }<pts>\n"
       << "\t{ -T, --title_size }<pts>\n"
       << "\t{ -L, --label_size }<pts>\n"
       << "\t{ -I, --icon_size }<pts>\n"
       << "\t{ -d, --default }           Show default values\n"
       << "\t{ -v, --version }\n"
       << "\t{ --<<media>> }        Where <<media>> is one of\n"
       << "\t `letter', `a4', `legal', `statement', `tabloid',\n"
       << "\t `ledger', `folio', `quarto', or `10x14'."
       << endl;
}

void defaults(char *foo){
  cout << foo 
       << " options have the following"
       << " default values or actions:\n\n"
       << "\t        Options \t Value\n\n"
       << "\t         output \t stdout\n"
       << "\t          title \n"
       << "\t         rotate \t no rotation\n"
       << "\t         labels \t labels are not printed\n"
       << "\t        noframe \t graph has frame\n"
       << "\t         height \t 11.0\n"
       << "\t          width \t 8.5\n"
       << "\t        margins \t 0.5\n"
       << "\t      linewidth \t 0.5\n"
       << "\t     elbow_room \t 0\n"
       << "\t   extra_margin \t 0\n"
       << "\t     title_size \t 18\n"
       << "\t     label_size \t 9\n"
       << "\t      icon_size \t 9\n"
       << "\t   If no <<media>> option is used,\n"
       << "\t   an EPS file will be produced\n"
       << endl;
  exit(0);
}

int main (int argc, char **argv)
{
  Plotparm p;
  char outfile[maxstr], pedfilename[maxstr];
  FILE *file;
  int kk;
  int openpedc;
  
  static struct option _options[] =
  {
    {"rotate", 0, 'r'},
    {"noids",0,'n'},
    {"labels", 0, 'l'},
    {"noframe", 0, 'f'},
    {"noicons", 0,'i'},
    {"height", 1,'H'},
    {"width", 1,'W'},
    {"linewidth", 1,'w'},
    {"margins", 1,'m'},
    {"output",0,'o'},
    {"title", 1,'t'},
    {"elbow_room", 1,'e'},
    {"extra_margin", 1,'x'},
    {"title_size", 1,'T'},
    {"label_size", 1,'L'},
    {"icon_size", 1,'I'},
    {"default",0,'d'},
    {"version",0,'v'},
    {"letter",0,'1'},
    {"a4",0,'2'},
    {"legal",0,'3'},
    {"statment",0,'4'},
    {"tabloid",0,'5'},
    {"ledger",0,'6'},
    {"folio",0,'7'},
    {"quarto",0,'8'},
    {"10x14",0,'9'},
    {0, 0, 0}
  };


  outfile[0]='\0';
      p.frame = YES;
      p.icons = YES;
      p.ids = YES;
      p.labels = NO;
      p.width = PAGEWIDTH;
      p.height = PAGEHEIGHT;
      p.margins = DEFAULTMARGIN;
      p.linewidth = DEFAULTLINEWIDTH;
      p.title[0] = 0;
      p.title_ptsize = DEFAULT_TITLE_PTSIZE;
      p.label_ptsize = DEFAULT_LABEL_PTSIZE;
      p.icon_size = DEFAULT_ICON_SIZE;
      p.elbow_room = DEFAULT_ELBOW_ROOM;
      p.extra_margin = 0.0;
      p.rotate = NO;
      p.media = 0;
      p.graphtype = MALEGG | FEMALEGG;
      GetOpt opto(argc, argv,"rnlfiH:W:w:m:o:t:e:x:T:L:I:p:a:vd123456789", 
		      _options);
    
  while (1){
    kk = opto();
    if (kk == -1)
      break;
      
    switch (kk)
      {
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
	p.media=kk - '1' + 1;
	break;
      case 'l':
	p.labels = YES;
	break;
      case 'n':
	p.ids = NO;
	break;
      case 'f':
	p.frame = NO;
	break;
     case 'd':
	defaults(argv[0]);
	break;
      case 'i':
	p.icons = NO;
	break;
      case 'r':
	p.rotate = YES;
	break;
      case 'H':
	if(! opto.optarg){ usage(argv[0]); exit(1);}
	p.height = atof(opto.optarg);
	break;
      case 'W':
	if(! opto.optarg){ usage(argv[0]); exit(1);}
	p.width = atof(opto.optarg);  
	break;
      case 'w':
	if(! opto.optarg){ usage(argv[0]); exit(1);}
	p.linewidth = atof(opto.optarg);  
	break;
      case 'm':
	if(! opto.optarg){ usage(argv[0]); exit(1);}
	p.margins = atof(opto.optarg);  
	break;
       case 't':
	if(! opto.optarg){ usage(argv[0]); exit(1);}
	strcpy(p.title,opto.optarg);
	break;
      case 'T':
	if(! opto.optarg){ usage(argv[0]); exit(1);}
	p.title_ptsize = atof(opto.optarg);
	break;
      case 'L':
	if(! opto.optarg){ usage(argv[0]); exit(1);}
	p.label_ptsize = atof(opto.optarg);
	break;
      case 'I':
	if(! opto.optarg){ usage(argv[0]); exit(1);}
	p.icon_size = atof(opto.optarg);
	break;
      case 'e':
	if(! opto.optarg){ usage(argv[0]); exit(1);}
	p.elbow_room = atof(opto.optarg);
	break;
      case 'x':
	if(! opto.optarg){ usage(argv[0]); exit(1);}
	p.extra_margin = atof(opto.optarg);
      case 'o':
	if(! opto.optarg){usage(argv[0]); exit(1);}
	strcpy(outfile,opto.optarg);
	break;
      case 'v':
	fprintf(stdout,"Version %s.\n",version);
	exit(0);
      case '?':
	usage(argv[0]);
	exit(1);
	break;
     
      default:
	usage(argv[0]);
	exit(0);
      }
  }

  if (opto.optind < argc){
    strcpy(pedfilename, argv[opto.optind++]);
    if ( (file = fopen(pedfilename,"r")) != NULL ){
      openpedc = inputpedc(file,0);
      fclose(file);
      if(!openpedc){
	cerr << "Error file: "<< pedfilename <<" is not in \"pedc\" format." << endl;
	exit(1);
      }
      ofstream fout;
      if(outfile[0] != '\0'){
	fout.open(outfile);
	if(!fout.is_open()){
	  cerr << "pedc2ps: cannot open "<< outfile << " for writting." << endl;
	  exit(4);
	}
	drawps(fout, argv[0], p);
	fout.close();
      }else
	drawps(argv[0], p);
    }
  }
  else{
    usage(argv[0]);
    exit(1);
  }     
  exit (0);
}

