#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Float_Input.H>
#include <FL/Fl_Round_Button.H>
#include <FL/Fl_Return_Button.H>
#include <FL/Fl_Choice.H>
#include <FL/fl_ask.H>

extern const char *fl_ok;
extern const char *fl_cancel;

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <fstream>
#include <sstream>
#include "postscript.h"
#include "headrandom.h"

using namespace std;

static int cero=0, uno=1, dos=2, tres=3, cuatro=4,
  cinco=5, seis=6, siete=7, ocho=8, nueve=9;
Fl_Menu_Item media[] ={
  {"default",0,0,(void *)&cero},
  {"letter",0,0,(void *)&uno},
  {"a4",0,0,(void *)&dos},
  {"legal",0,0,(void *)&tres},
  {"statement",0,0,(void *)&cuatro},
  {"tabloid",0,0,(void *)&cinco},
  {"ledger",0,0,(void *)&seis},
  {"folio",0,0,(void *)&siete},
  {"quarto",0,0,(void *)&ocho},
  {"10x14",0,0,(void *)&nueve},
  {0}
};

Fl_Float_Input *oh, *ow;
Fl_Input *cmd;
class PrintPS : public Fl_Window{
private:
//  float width, height, margin, linewidth, title_size,
//    label_size, icon_size, elbow_room, extra_margin;
public:
  Fl_Input *title;
  Fl_Round_Button *opt1[5];
  Fl_Float_Input *opt2[7];
  Fl_Button *ok_;
  Fl_Button *cancel_;
  Fl_Choice *medium;
  PrintPS();
};

void cb_medium(Fl_Widget *co){
  switch(((Fl_Choice *)co)->value()){
  case 1:
    oh->value("11.0");
    ow->value("8.5");
    break;
  case 2:
    oh->value("11.69");
    ow->value("8.26");
    break;
  case 3:
    oh->value("14.0");
    ow->value("8.5");
    break;
  case 4:
    oh->value("8.5");
    ow->value("5.5");
    break;
  case 5:
    oh->value("17.0");
    ow->value("11.0");
    break;
  case 6:
    oh->value("11.0");
    ow->value("17.0");
    break;
  case 7:
    oh->value("13.0");
    ow->value("8.5");
    break;
  case 8:
    oh->value("10.83");
    ow->value("8.5");
    break;
  case 9:
    oh->value("14.0");
    ow->value("10.0");
    break;
  }
}

PrintPS::PrintPS() : Fl_Window(400,410)
{
  int i;
  begin();
  title = new Fl_Input(72,320,293,30,"title: ");
  cmd = new Fl_Input(155,10,210,30,"printer command: ");
  cmd->value("lpr");

  opt1[0] = new Fl_Round_Button(35,100,95,20,"frame");
  opt1[1] = new Fl_Round_Button(35,125,95,20,"icons");
  opt1[2] = new Fl_Round_Button(35,150,95,20,"identity");
  opt1[3] = new Fl_Round_Button(35,175,95,20,"labels");
  opt1[4] = new Fl_Round_Button(35,200,95,20,"landscape");

  for(i=0; i<5; i++){
    opt1[i]->box(FL_UP_BOX);
    opt1[i]->down_box(FL_ROUND_DOWN_BOX);
    opt1[i]->set();
  }
  opt1[3]->clear(); opt1[4]->clear();
  oh = new Fl_Float_Input(300,57,65,30,"height: ");
  oh->value("11.0");
  ow = new Fl_Float_Input(300,92,65,30,"width: ");
  ow->value("8.5");
  opt2[0] = new Fl_Float_Input(300,127,65,30,"margins: ");
  opt2[0]->value("0.5");
  opt2[1] = new Fl_Float_Input(300,162,65,30,"title size: ");
  opt2[1]->value("18");
  opt2[2] = new Fl_Float_Input(300,197,65,30,"text size: ");
  opt2[2]->value("9");
  opt2[3] = new Fl_Float_Input(300,232,65,30,"icon size: ");
  opt2[3]->value("9");
  opt2[4] = new Fl_Float_Input(300,267,65,30,"line width: ");
  opt2[4]->value("0.5");
  opt2[5] = new Fl_Float_Input(126,232,65,30,"elbow room: ");
  opt2[5]->value("0");
  opt2[6] = new Fl_Float_Input(126,267,65,30,"extra margin: ");
  opt2[6]->value("0");
  medium = new Fl_Choice(95,57,80,25,"medium: ");
  medium->menu(media);
  medium->when(FL_WHEN_CHANGED);
  medium->callback(cb_medium);
  ok_ = new Fl_Return_Button(230,370,100,25,fl_ok);
  cancel_ = new Fl_Button(70,370,100,25,fl_cancel);
  end();
  set_modal();
}


void print_it(){
  static PrintPS *prn;
  if(!prn) prn = new PrintPS();
  prn->ok_->label(fl_ok);
  prn->cancel_->label(fl_cancel);
  prn->show();
  int ok=0;
  do{
    Fl::wait();
    Fl_Widget *o = Fl::readqueue();
    if(o == prn->ok_){ ok = 1; break;}
    if(o == prn->cancel_) break;
  }while(1);
  prn->hide();
  if(ok){
    Plotparm p;
    p.frame = prn->opt1[0]->value();
    p.icons = prn->opt1[1]->value();
    p.ids =  prn->opt1[2]->value();
    p.labels = prn->opt1[3]->value();
    p.rotate = prn->opt1[4]->value();
    p.height = atof(oh->value());
    p.width = atof(ow->value());
    p.margins = atof(prn->opt2[0]->value());
    p.title_ptsize = atof(prn->opt2[1]->value());
    p.label_ptsize = atof(prn->opt2[2]->value());
    p.icon_size = atof(prn->opt2[3]->value());
    p.linewidth = atof(prn->opt2[4]->value());
    p.elbow_room = atof(prn->opt2[5]->value());
    p.extra_margin = atof(prn->opt2[6]->value());
    p.media = prn->medium->value();
    if(strlen(prn->title->value()) > 0)
      strcpy(p.title,prn->title->value());
    else
	p.title[0]='\0';
    ostringstream tmp;
    tmp << "pfiddler" << randint(10000, 64000) << ".ps";
    ofstream fout((tmp.str()).c_str());
    if(fout.is_open()){
      drawps(fout, "pfiddler",p);
      fout.close();
      char * prncmd= new char[256];
      strcpy(prncmd,cmd->value());
      strcat(prncmd," ");
      strcat(prncmd,(tmp.str()).c_str());
      system(prncmd);
      remove((tmp.str()).c_str());
      delete [] prncmd;
    }else
      fl_alert("Unable to open temporary buffer");
  }
}

class doEPS : public Fl_Window{
private:
//  float width, height, margin, linewidth, title_size,
//    label_size, icon_size, elbow_room, extra_margin;
public:
  Fl_Input *epsfile;
  Fl_Input *title;
  Fl_Round_Button *opt1[5];
  Fl_Float_Input *opt2[9];
  Fl_Button *ok_;
  Fl_Button *cancel_;
  doEPS();
};


doEPS::doEPS() : Fl_Window(400,410)
{
  int i;
  begin();
  title = new Fl_Input(72,320,293,30,"title: ");
  epsfile = new Fl_Input(155,10,210,30,"file name: ");
  epsfile->value("out.eps");
  opt1[0] = new Fl_Round_Button(35,65,95,25,"frame");
  opt1[1] = new Fl_Round_Button(35,95,95,25,"icons");
  opt1[2] = new Fl_Round_Button(35,125,95,25,"identity");
  opt1[3] = new Fl_Round_Button(35,155,95,25,"labels");
  opt1[4] = new Fl_Round_Button(35,185,95,25,"rotate");

  for(i=0; i<5; i++){
    opt1[i]->box(FL_UP_BOX);
    opt1[i]->down_box(FL_ROUND_DOWN_BOX);
    opt1[i]->set();
  }
  opt1[3]->clear();  opt1[4]->clear();
  opt2[0] = new Fl_Float_Input(300,57,65,30,"height: ");
  opt2[0]->value("11.0");
  opt2[1] = new Fl_Float_Input(300,92,65,30,"width: ");
  opt2[1]->value("8.5");
  opt2[2] = new Fl_Float_Input(300,127,65,30,"margins: ");
  opt2[2]->value("0.5");
  opt2[3] = new Fl_Float_Input(300,162,65,30,"title size: ");
  opt2[3]->value("18");
  opt2[4] = new Fl_Float_Input(300,197,65,30,"text size: ");
  opt2[4]->value("9");
  opt2[5] = new Fl_Float_Input(300,232,65,30,"icon size: ");
  opt2[5]->value("9");
  opt2[6] = new Fl_Float_Input(300,267,65,30,"line width: ");
  opt2[6]->value("0.5");
  opt2[7] = new Fl_Float_Input(126,232,65,30,"elbow room: ");
  opt2[7]->value("0");
  opt2[8] = new Fl_Float_Input(126,267,65,30,"extra margin: ");
  opt2[8]->value("0");

  ok_ = new Fl_Return_Button(230,370,100,25,fl_ok);
  cancel_ = new Fl_Button(70,370,100,25,fl_cancel);
  end();
  set_modal();
}


void eps_it(){
  static doEPS *prn;
  if(!prn) prn = new doEPS();
  prn->ok_->label(fl_ok);
  prn->cancel_->label(fl_cancel);
  prn->show();
  int ok=0;
  do{
    Fl::wait();
    Fl_Widget *o = Fl::readqueue();
    if(o == prn->ok_){ ok = 1; break;}
    if(o == prn->cancel_) break;
  }while(1);
  prn->hide();
  if(ok){
    Plotparm p;
    p.frame = prn->opt1[0]->value();
    p.icons = prn->opt1[1]->value();
    p.ids = prn->opt1[2]->value();
    p.labels = prn->opt1[3]->value();
    p.rotate = prn->opt1[4]->value();
    p.height = atof(prn->opt2[0]->value());
    p.width = atof(prn->opt2[1]->value());
    p.margins = atof(prn->opt2[2]->value());
    p.title_ptsize = atof(prn->opt2[3]->value());
    p.label_ptsize = atof(prn->opt2[4]->value());
    p.icon_size = atof(prn->opt2[5]->value());
    p.linewidth = atof(prn->opt2[6]->value());
    p.elbow_room = atof(prn->opt2[7]->value());
    p.extra_margin = atof(prn->opt2[8]->value());
    p.media = 0;
    if(strlen(prn->title->value()) > 0)
      strcpy(p.title,prn->title->value());
    else
	p.title[0]='\0';
    ofstream fout(prn->epsfile->value());
    if(fout.is_open()){
      drawps(fout, "pfiddler",p);
      fout.close();

    }else
      fl_alert("Unable to open file for writting.");
  }
}
