// txt2pedc.cc
#include <cstdlib>
#include "txt2pedc.h"
#include <iostream>
#include <Fl/fl_ask.H>
using namespace std;

bool _sex_flag;
bool _status_flag;
bool _label_flag;
bool _console_ = 0;


inline istream & operator >>(istream &s, rawpedrecord &p)
{
  char buff[1025];
  char kid[128];
  char fa[128];
  char mo[128];
  int i;
  int x1, x2;
  s.getline(buff,1024);	
  istringstream in(buff);
  in >> kid >> fa >> mo;
  if(_sex_flag) in >> x1;
  else x1=0;
  if(_status_flag) in >> x2;
  else x2=0;
  if(_label_flag){
    i = in.tellg();
    if(i == -1){
      i=0; buff[0] = 0;
    }
    while(buff[i] == ' ' || buff[i] == '\t') i++;
  }else{
    for(i = 0; buff[i] != ' '  && buff[i] != '\t' && buff[i] != 0; i++);
    buff[i] = 0; i = 0;
  }  
  p.update(kid,fa,mo,x1,x2,&buff[i]);
  return s;
}

inline ostream & operator <<(ostream &s, const rawpedrecord &p)
{
  s << p.id << ' '
    << p.dad << ' '
    << p.mom;
  if(_sex_flag)
    s << ' ' << p.sex;
  if(_status_flag)
    s << ' ' << p.status;
  if(_label_flag)
    s << ' ' << p.label; 
  s << endl;
  return s;
}

inline ostream & operator <<(ostream &s, const pedigree &p)
{
  for(unsigned int i=1; i <= p.N; i++){
    s << i << ' '
      << p.father[i] << ' '
      << p.mother[i];
    if(_sex_flag)
      s << ' ' << p.sex[i];
    if(_status_flag)
      s << ' ' << p.status[i];
    s << ' ' << p.id[i];
    if(_label_flag)
      s << ' ' << p.label;
    s << endl;
  }
  return s;
}

rawpedrecord::rawpedrecord(char *Id, char *Dad, char *Mom, int _sex, int _status, char *lab){
    id = new char[strlen(Id)+1];
    strcpy(id,Id);
    if(Dad != NULL){
       dad = new char[strlen(Dad)+1];
    strcpy(dad,Dad);
    }
    if(Mom != NULL){
       mom = new char[strlen(Mom)+1];
    strcpy(mom,Mom);
    }
    sex = _sex;
    status = _status;
    if(lab != NULL){
       label = new char[strlen(lab)+1];
    strcpy(label,lab);
    }
  }

void rawpedrecord::update(char *Id, char *Dad, char *Mom, int _sex, int _status,char *lab){
  if(id != NULL) delete [] id;
  if(mom != NULL){ delete [] mom; mom = NULL;}
  if(dad != NULL){ delete [] dad; dad = NULL;}
  if(label != NULL){ delete [] label; label = NULL;}
    id = new char[strlen(Id)+1];
    strcpy(id,Id);
    if(Dad != NULL){
       dad = new char[strlen(Dad)+1];
    strcpy(dad,Dad);
    }
    if(Mom != NULL){
       mom = new char[strlen(Mom)+1];
    strcpy(mom,Mom);
    }
    sex = _sex; status = _status;
    if(lab != NULL){
      label = new char[strlen(lab)+1];
      strcpy(label,lab);
    }
}

void check1(const int n, const rawpedrecord *p){
  for(int i = 0; i<n; i++){
    if(strcmp(p[i].id,p[i].dad) == 0 || strcmp(p[i].id, p[i].mom) == 0){
      if(_console_)
	cerr << "ERROR: '" << p[i].id << "' cannot be its own parent." <<  endl;
      else
	fl_alert("ERROR: '%s' cannot be its own parent.",p[i].id);
      exit(1);
    }
  }
  for(int n1 = n-1, i=0; i<n1; i++)
    for(int j=i+1; j<n; j++)
      if(strcmp(p[i].id, p[j].id) == 0){
	if(_console_)
	  cerr << "ERROR: '" << p[i].id << "' is not unique." <<  endl;
	else
	  fl_alert("ERROR: '%s' is not unique.", p[i].id);
	exit(1);
      }
} 

void FixPrecedence(const int n, rawpedrecord *p)
{
  check1(n,p);
  rawpedrecord tmp;
  int i,j, n_1, jo;
  n_1 = n-1;
  for(i=0; i <n_1 ; i++){
    jo = 2*(n-i);
    for(j=i+1; j < n; j++){
      if(strcmp(p[i].dad, p[j].id) == 0){
	if(!jo--){
	  if(_console_)
	    cerr << "ERROR: '" << p[j].id << "' cannot be its own ancestor."<<endl;
	  else
	    fl_alert("ERROR: '%s' cannot be its own ancestor.",p[j].id);
	  exit(1);
	}
	tmp = p[j];
	p[j] = p[i];
	p[i] = tmp;
	j = i;
	continue;
      }
      if(strcmp(p[i].mom, p[j].id) == 0){
	if(!jo--){
	  if(_console_)
	    cerr << "ERROR: '" <<p[j].id << "' cannot be its own ancestor." <<endl;
	  else
	    fl_alert("ERROR: '%s' cannot be its own ancestor.",p[j].id);
	  exit(1);
	}       
	tmp = p[j];
	p[j] = p[i];
	p[i] = tmp;
	j = i;
      }
    }
  }
}

void GetRawPedigree(ifstream &in, const int n, rawpedrecord *data)
{
  int i;
  for(i=0; i<n; i++)
    in >> data[i];
}

/*
void PrintRawPedigree(const int n, const rawpedrecord *data)
{
  int i;
  for(i=0; i<n; i++)
    cout << data[i];
}
*/

void pedigree::setup(const unsigned int n, const unsigned int d){
  N=0;
  Nvec = (n > 10 ? n : 10);
  Dvec = (d > 10 ? d : 10);
  id=new char *[Nvec];
  for(unsigned int i=0; i < Nvec; i++) id[i] = NULL;
  father = new unsigned int [Nvec];
  mother = new unsigned int [Nvec];
  sex = new int [Nvec];
  status = new int [Nvec];
  label = new char *[Nvec];
  for(unsigned int i=0; i < Nvec; i++) label[i] = NULL;
  id[0] = new char[2];
  id[0][0] = '0'; id[0][1]='\0';
  father[0]=0; mother[0]=0;
  sex[0]=0; status[0] = 0;
}

pedigree::~pedigree(){
  unsigned int i;
  for(i=0; i<= N; i++){
    if(id[i]) delete [] id[i];
    if(label[i]) delete [] label[i];
  }
  delete [] id;
  delete [] father;
  delete [] mother;
  delete [] sex;
  delete [] status;
  delete [] label;
}

bool isnotfounder(const char *parent){
  if(parent){
    const char *p = parent;
    while(*p == ' ') p++;
    if(p[0] == '\0' || 
       (p[0] == '0' && p[1] == '\0')) return 0;
  }
  return 1;
}

void pedigree::append(const char *Id, const char *Fa, const char *Mo,
		      const int &_sex, const int &_status, const char *lab){
  unsigned int i,j;
  if((i = isnotfounder(Fa))){
    for(; i<= N; i++) 
      if(strcmp(Fa,id[i]) == 0) break;
    if(i > N)
      add(Fa,0,0,1,0,"");
  }
  if((j = isnotfounder(Mo))){
    for(; j<= N; j++) 
      if(strcmp(Mo,id[j]) == 0) break;
    if(j > N)
      add(Mo,0,0,2,0,"");
  }
  add(Id, i, j, _sex, _status,lab);
}

void pedigree::add(const char *Id, const unsigned int Fa,
		   const unsigned int Mo, const int _sex, const int _status,
		   const char *lab){
    N++;
  if(N == Nvec){
    Nvec += Dvec;
    char **tmp1 = new char *[Nvec];
    unsigned int *tmp2 = new unsigned int [Nvec];
    unsigned int *tmp3 = new unsigned int [Nvec];
    int *tmp4 = new int [Nvec];
    int *tmp5 = new int [Nvec];
    char **tmp6 = new char *[Nvec];

    unsigned int i;
    for(i=0; i< N; i++){
      tmp1[i] = id[i];
      tmp2[i] = father[i];
      tmp3[i] = mother[i];
      tmp4[i] =  sex[i];
      tmp5[i] = status[i];
      tmp6[i] = label[i];
    }
    delete [] id;
    delete [] father;
    delete [] mother;
    delete [] sex;
    delete [] status;
    delete [] label;
    id = tmp1;
    father = tmp2;
    mother = tmp3;
    sex = tmp4;
    status = tmp5;
    label = tmp6;
    for(i=N; i< Nvec; i++){
     id[i] = NULL;
     label[i] = NULL;
     }
  }
  id[N]= new char[strlen(Id)+1];
  strcpy(id[N],Id);
  father[N]=Fa;
  mother[N]=Mo;
  sex[N] = _sex;
  status[N] = _status;
  label[N] = new char[strlen(lab)+1];
  strcpy(label[N],lab);
}

pedigree::pedigree(unsigned int n, const char **Id, const char **Dad,
		   const char **Mom, const int *_sex, const int *_status,
		   const char **lab){
  setup( n + 100 );
  for(unsigned int i=0; i<n; i++)
    append( Id[i], Dad[i], Mom[i], _sex[i], _status[i], lab[i] );
}


pedigree::pedigree(const unsigned int n, const rawpedrecord *rec){
  setup( n + 100 );
  for(unsigned int i=0; i< n; i++)
    append( rec[i].id, rec[i].dad, rec[i].mom, rec[i].sex, rec[i].status,
            rec[i].label );
}

pedigree::pedigree(const char *filename){
  ifstream fp;  
  fp.open(filename);
  if(!fp.eof()){
    rawpedrecord *rec = new rawpedrecord[1];
    unsigned int i,n;
    n = 0;
    while(!fp.eof()){
      if( !(n %100) ){
	rawpedrecord *tmp = rec;
	rec = new rawpedrecord[n+100];
	for(i=0; i<n; i++)
	  rec[i] = tmp[i];
	delete [] tmp;
      }
      fp >> rec[n];
      n++;
    }
    n--;
    fp.close();
    FixPrecedence(n,rec);
    setup( n + 100 );
    for(i=0; i< n; i++)
      append( rec[i].id, rec[i].dad, rec[i].mom, rec[i].sex, rec[i].status,
              rec[i].label );
    delete [] rec;
  }
}


#include <cstdio>

int numlines(const char *, int &);

#define MAXNGEN 256
#include "headio.h"
#include "Headped.H"


extern "C" {
  int setindc(int,int,int,int,unsigned char,
		     unsigned char,unsigned char,char *, char *);
  void initpedc(int,int,int,int,int);
  void outputpedc(FILE *, char *,int);
  void mngrcoords();
  void annealcoords(int, float, float);
}

void readlist_ (const char *filename,int sexes, int affection,int labels,
		int rn, int gn, int bn, int rd, int gd, int bd,
		int ru, int gu, int bu)
{
  int rows, cols;
  int ierror = 1;
  _sex_flag =sexes;
  _status_flag=affection;
  _label_flag=labels;
  unsigned char r,g,b;
  rows = numlines(filename, cols);
  pedigree _ped(filename);

  initpedc(2*_ped.N+1,_ped.N+1,2*_ped.N+1,_ped.N+1,2*_ped.N+1);
  
  for(unsigned int j=1; j <=_ped.N; j++){
    switch(_ped.status[j]){
    case 2:
      r=rd; g=gd; b=bd;
      break;
    case 0:
      r=ru; g=gu; b=bu;
      break;
    default:
      r=rn; g=gn; b=bn;
    }
    ierror &= setindc(j,_ped.father[j],_ped.mother[j],
		      _ped.sex[j],r,g,b,_ped.id[j],_ped.label[j]);
  }
  if(!ierror){ 
    if(_console_)
      cerr << "Pedigree indexing error." << endl;
    else
      fl_alert("Pedigree indexing error.");
    exit(1);
  }
  pedset=1;
  for (int i=1; i<=indpt; i++) indstack[i].info.flag = UP;
  for (int i=1; i<=marpt; i++) marstack[i].info.flag = UP;
  mngrcoords();
  coordset =1;
}

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Return_Button.H>
#include <FL/Fl_Check_Button.H>
#include <Fl/fl_draw.h>
#include <Fl/Fl_Tooltip.H>
#include <Fl/Fl_Color_Chooser.H>
#include <Fl/Fl_File_Chooser.H>
#include <Fl/Fl_File_Icon.H>
extern const char *fl_ok;
extern const char *fl_cancel;

struct myrgb{
  unsigned char r;
  unsigned char g;
  unsigned char b;
};

class Txt2Pedc : public Fl_Window{
public:
  myrgb *n, *a, *u;
  Fl_Button *ok_;
  Fl_Button *cancel_;
  Fl_Button *nor;
  Fl_Button *aff;
  Fl_Button *unk;
  Fl_Input *txtname;
  Fl_Button *txt_;
  Fl_Button *pedc_;
  Fl_Input *pedcname;
  Fl_Check_Button *sex_;
  Fl_Check_Button *afsta_;
  Fl_Check_Button *labels_;
  Txt2Pedc();
};

void mycolor_cb(Fl_Widget *b, void *v){
  myrgb *c = (myrgb *)v;
  if(fl_color_chooser("Icon color",c->r,c->g,c->b))
    b->color(fl_rgb_color(c->r,c->g,c->b));
}

Txt2Pedc::Txt2Pedc() : Fl_Window(325,350){
  Fl_Box *msg;
  begin();
  a = new myrgb;
  n = new myrgb;
  u = new myrgb;
  n->r = n->g = n->b = u->r = 255;
  a->g = a->r = 25;
  a->b = 112;
  u->g = 222;
  u->b = 173;
  msg = new Fl_Box(25,100,137,25,"Optional information:");
  msg = new Fl_Box(25,210,165,35,"Affection status color:");
  ok_ = new Fl_Return_Button(200,300,100,25,fl_ok);
  cancel_ = new Fl_Button(75,300,100,25,fl_cancel);
  txtname = new Fl_Input(100,25,170,25,"Input name:");
  pedcname = new Fl_Input(100,55,170,25,"Output name:");
  sex_ = new Fl_Check_Button(25,125,50,25,"Sex");
  afsta_ = new Fl_Check_Button(91,127,125,25,"Affection status");
  labels_ = new Fl_Check_Button(235,125,68,25,"Labels");
  nor = new Fl_Button(200,170,75,35,"Normal");
  aff = new Fl_Button(200,210,75,35,"Affected");
  unk = new Fl_Button(200,250,75,35,"Unknown");
  sex_->box(FL_UP_BOX);
  sex_->set();
  afsta_->box(FL_UP_BOX);
  afsta_->set();
  labels_->box(FL_UP_BOX);
  labels_->set();
  nor->tooltip("Normal");
  aff->tooltip("Affected");
  unk->tooltip("Unknown");
  nor->callback(mycolor_cb, (void *)n);
  aff->callback(mycolor_cb, (void *)a);
  unk->callback(mycolor_cb, (void *)u);
  nor->color(fl_rgb_color(n->r,n->g,n->b));
  aff->color(fl_rgb_color(a->r,a->g,a->b));
  unk->color(fl_rgb_color(u->r,u->g,u->b));
  txtname->value("input.txt");
  pedcname->value("output.pedc");
  txt_ = new Fl_Button(275,25,25,25);
  pedc_ = new Fl_Button(275,55,25,25);
  end();
  set_modal();
}

char filename[maxline];


bool txt2pedc_it(char *pedname){
  bool ok = false;
  char *fp;
  Fl_File_Icon::load_system_icons();
  Fl_File_Icon *icon_dir;
  icon_dir = Fl_File_Icon::find("*",Fl_File_Icon::DIRECTORY);
  Txt2Pedc *sa = new Txt2Pedc;
  sa->ok_->label(fl_ok);
  sa->cancel_->label(fl_cancel);
  icon_dir->label(sa->txt_);
  icon_dir->label(sa->pedc_);

  sa->show();
  do{
    Fl::wait();
    Fl_Widget *o = Fl::readqueue();
    if(o == sa->txt_){
      if((fp = fl_file_chooser("Input file","*.txt",NULL)))
	sa->txtname->value(fp);
    }
    if(o == sa->pedc_){
      if((fp = fl_file_chooser("Output file","*.pedc",NULL)))
 	sa->pedcname->value(fp);
    }
    if(o == sa->ok_){ ok = true; break;}
    if(o == sa->cancel_) break;
  }while(1);
  sa->hide();
  if(ok){
    readlist_(sa->txtname->value(),sa->sex_->value(),sa->afsta_->value(),
	      sa->labels_->value(),sa->n->r,sa->n->g,sa->n->b,
	      sa->a->r,sa->a->g,sa->a->b,sa->u->r,sa->u->g,sa->u->b);
    strcpy(pedname,sa->pedcname->value());
  }
#ifdef WITHMAIN
  else
    exit(1);
#endif
  return(pedset ? true : false);
}


#ifdef WITHMAIN

void readlist_ (char *filename,int sexes, int affection,int labels,
		    char *nor, char *dis, char *unk){
  int rn, rd, ru, gn, gd, gu, bn, bd, bu;
  sscanf(nor,"%d,%d,%d",&rn,&gn,&bn);
  sscanf(dis,"%d,%d,%d",&rd,&gd,&bd);
  sscanf(unk,"%d,%d,%d",&ru,&gu,&bu);
  readlist_ (filename,sexes,affection,labels,rn,gn,bn,rd,gd,bd,ru,gu,bu);
}

#include "getopti.h"
extern "C"{
#include "util.h"
}

static char version[]="0.6 (20070304) JC Loredo-Osti and K Morgan.";


void usage(char *name){
  cout << "\n     Usage: " << name << " [options] <file>\n\n"
       << "   Transforms a list in a pedigree coordinates file.\n"
       << "   List columns are:\n"
       << "     <id> <father> <mother> [sex] [affection_status] [label]\n"
       << "         <> and [] mean mandatory and optional fields, respectively.\n"
       << "Options:\n"
       << "\t{ -o | --output }<file>     Output file\n"
       << "\t{ -s | --sex }              Input file includes sex\n"
       << "\t{ -a | --affection }        Input file includes affection status\n"
       << "\t{ -l | --labels }           Input file includes labels\n"
       << "\t{ -m | --minimize }         Minimize the square of line distances\n"
       << "\t                                 by simulated annealing\n"
       << "\t{ -I | --iterations }<n>    Iterations for annealing\n"
       << "\t{ -T | --temperature }<f>   Annealing starting temperature\n"
       << "\t{ -C | --cooling }<f>       Annealing cooling factor\n"
       << "\t                            RGB color for the affected status:\n"
       << "\t{ -N | --normal }<r,g,b>         0 < r,g,b <= 255\n"
       << "\t{ -A | --affected }<r,g,b>       without spaces between numbers\n"
       << "\t{ -U | --unknown }<r,g,b>\n"
       << "\t{ -d | --default }          Show default values\n"
       << "\t{ -v | --version }" << endl;
}

void defaults(char *name){
  cout << name
       << "\n  options have the following default values or actions:\n\n"
       << "\t        Option \t Value\n\n"
       << "\t        output \t stdout\n"
       << "\t           sex \t no sex information\n"
       << "\t     affection \t no affection status information\n"
       << "\t        labels \t labeling information ignored\n"
       << "\t      minimize \t no minimization\n"
       << "\t    iterations \t 1000\n"
       << "\t   temperature \t 15.0\n"
       << "\t       cooling \t 0.9\n"
       << "\t               \t RGB color for the affected status:\n"
       << "\t        normal \t 255,255,255   (white)\n"
       << "\t      affected \t 25,25,112     (midnight blue)\n" 
       << "\t       unknown \t 255,222,173   (Navajo white)\n" << endl;
  exit(0);
}

#include <cstdlib>

void console_main(int argc, char **argv, char *pedname)
{
  int annealing=0;
  int sexes = 0;
  int kk, tmp=0;
  int nit=1000;
  float boil;
  float cool;
  int affection = 0;
  char normal[65];
  char affected[65];
  char unknown[65];
  int labe = 0;
  static struct option _options[] = 
  {    
    {"output", 1,'o'},
    {"sex", 0,'s'},
    {"affection", 0,'a'},
    {"minimize",0,'m'},
    {"iterations",1,'I'},
    {"temperature",1,'T'},
    {"cooling",1,'C'},
    {"normal",1,'N'},
    {"affected",1,'A'},
    {"labels",0,'l'},
    {"unknown",1,'U'},
    {"default",1,'d'},
    {"version",0, 'v'},
    {"help",0,'h'},
    {0,0,0}
  };
  boil = 15.0;
  cool = 0.9;
  pedname[0]='\0';
  strcpy(normal,"255,255,255");
  strcpy(affected,"25,25,112");
  strcpy(unknown,"255,222,173");
  _console_ = 1; 
  GetOpt opto(argc, argv, "hlsamo:I:T:C:vr:N:A:U:d", _options);
  
  while (1){
    kk = opto();
    if (kk == -1)
      break;
    switch (kk){
      case 'h':
	usage(argv[0]);
	exit(1);
	break;
      case 'I':
	if(opto.optarg){
	  tmp = atoi(opto.optarg);
	  if(tmp > 0) nit = tmp;
	}
	break;
      case 'T':
	if(opto.optarg){
	  boil = atof(opto.optarg);
	}
	break;
      case 'C':
	if(opto.optarg){
	  cool = atof(opto.optarg);
	}
	break;
      case 'N':
	if(opto.optarg){
	  strcpy(normal,opto.optarg);
	}
	break;
      case 'A':
	if(opto.optarg){
	  strcpy(affected,opto.optarg);
	}
	break;
      case 'U':
	if(opto.optarg){
	  strcpy(unknown,opto.optarg);
	}
	break;
      case 'm':
	annealing = 1;
	break;
      case 'd':
	defaults(argv[0]);
	break;
      case 'a':
	affection = 1;
	break;
      case 'l':
	labe = 1;
	break;
      case 's':
	sexes = 1;
	break;
      case 'o':
	if(! opto.optarg){ usage(argv[0]); exit(1);}
	strcpy(pedname,opto.optarg);
	break;
      case 'v':
	cout << "Version " << version << endl;
	exit(0);
      default : 
	exit(1);
      }
  }

  if(opto.optind < argc)
    strcpy(filename, argv[opto.optind++]);
  else{
    usage(argv[0]);
    exit(1);
  }

  readlist_(filename,sexes,affection,labe,normal,affected,unknown);

  if (annealing)   annealcoords(nit,boil,cool);

}

int main(int argc, char **argv){
  char pedname[maxstr];
  FILE *outfile;
  if(argc > 1)
    console_main(argc,argv,pedname);
  else
    txt2pedc_it(pedname);
  if(pedname[0] != '\0'){
    if ( (outfile = fopen(pedname,"w")) == NULL ){ 
      cerr << "Failure to open output file " << pedname << endl
	   <<"Check write status of directory." << endl;
      exit(1);
    }
    else{
      outputpedc(outfile,pedname,0);
      fclose(outfile);
    }    
  }
  else{
    outputpedc(stdout,filename,0);
    fflush(stdout);
  }  

  return Fl::run(); 
}

#endif
